#!/usr/bin/perl

#
# uses ddmap to display the bits as a bitmap; each bit represents a 16K block that
# has changed
#

use strict;
use Data::Dumper;
use Tk;
use Tk::Canvas;

if ( !defined($ARGV[0]) )
{
	print "$0 .ddmap\n";
	exit(1);
}
my $ddmap_file = $ARGV[0];

#
# main window
#
my $w_main = MainWindow->new();
$w_main->title("ddmap: $ddmap_file");

my $max_x = 1024;
my $max_y = 0;
my $edge_offset = 0;
my $scroll_region = [0,0,$max_x,$max_y];

my $w_canvas = $w_main->Scrolled(
	"Canvas",
	-background => 'black',
	-scrollbars => 'ser',
	-width => $max_x + $edge_offset + 4,
	-height => 600,
	-scrollregion => $scroll_region,
	)->pack(
		-fill => 'both',
		-expand => 1,
	);

$w_main->update();
my $bitmap = read_ddmap($ddmap_file);
print "data ready... ";
draw($w_canvas, $bitmap);
print "map ready.\n";
Tk::MainLoop();
exit(0);

#0000000 642a 6d64 7061 002a 0001 0000 2911 0000
#0000010 4000 0006 d000 0000 0000 0000 0000 0000
#0000020 0000 0000 0000 0000 0000 0000 0000 0000

# the header is processed as if it was bits
sub read_ddmap
{
	my ($file) = @_;
	
	my @bitmap;
	my $bidx = 0;
	my $buffer;
	open(FH, "<$file") or die "unable to read: $!";
	while( read(FH, $buffer, 1024) )
	{
		my $length = length($buffer);
		my $words = $length / 4;
		
		my @data = unpack "L[$words]", $buffer;

		foreach my $datum (@data)
		{
			#printf "%08x:", $datum;
			for(my $i=0; $i<32; $i++)
			{
				#printf "%02d %08x & %08x: %d\n", $i, $datum, (2**$i), $datum & (2**$i);
				if ( $datum & (2 ** $i) )
				{
					#print "1";
					$bitmap[$bidx] = 1;
				}
				$bidx++;
				#else
				#{
				#	print "0";
				#}
			}
			#print " ";
		}
		#print "\n";
	}
	return \@bitmap;
}

sub draw
{
	my ($w_canvas, $bitmap) = @_;
	
	my $y = 0;
	my $x = 0;
	for(my $bidx = 0; $bidx < scalar(@{$bitmap}); $bidx++)
	{
		my $xs = $edge_offset + $x*1;
		my $ys = $edge_offset + $y*1;
		#print "$xs,$ys\n";
		if ( $$bitmap[$bidx] )
		{
			my $color = "#00cc00";
			#my $color = sprintf("#%02x%02x%02x",
			#	0, $$bitmap[$bidx] * (256/$combine_segments) - 1,0);
				
			#print "$color\n";
			$w_canvas->createRectangle( 
				$xs,
				$ys,
				$xs+1,
				$ys+1,
				-fill => $color,
				-width => 0,
			);
		}
		
		
		$x++;
		if ( $x>$max_x )
		{
			$x=1;
			$y++;

			$$scroll_region[3] = $y;
			$w_canvas->configure(-scrollregion => $scroll_region);
			
			if ( $y % 10 == 0 )
			{
				$w_main->update();
			}
		}
	}
}
