#!/bin/sh

#
# iptables control script
#
# Steffen Plotner, 2010
#

. /etc/iscsi-target/ietd_config.sh

#------------------------------------------------------------------------------
# start
#------------------------------------------------------------------------------
function start()
{
	local ip_spec="$*"
	
	verbose=0
	if [ -z "$ip_spec" ]; then
		ip_spec="0.0.0.0/0"
		verbose=1
	fi

	if [ $verbose -eq 1 ]; then
		echo -n "Starting iSCSI Target: iptables"
		sleep 1
	fi

	#
	# allow iscsi traffic in and out
	#
	iptables_unblock_iscsi "$ip_spec"

	if [ $verbose -eq 1 ]; then
		echo_success
		echo
	fi
}

#------------------------------------------------------------------------------
# stop
#------------------------------------------------------------------------------
function stop()
{
	local ip_spec="$*"

	verbose=0
	if [ -z "$ip_spec" ]; then
		ip_spec="0.0.0.0/0"
		verbose=1
	fi

	if [ $verbose -eq 1 ]; then
		echo -n "Stopping iSCSI Target: iptables"
	fi

	#
	# disallow iscsi traffic in/out
	#
	iptables_block_iscsi "$ip_spec"
	
	if [ $verbose -eq 1 ]; then
		echo_success
		echo
	fi
}

#------------------------------------------------------------------------------
# status
#------------------------------------------------------------------------------
function status()
{
	iptables -L -n
}

case "$1" in
	start)
		shift
		start $*
		;;
	stop)
		shift
		stop $*
		;;
	status)
		status
		;;
	*)
		echo $"Usage: $0 {start|stop|status}"
		exit 1
esac
exit 0
