#!/bin/sh

#------------------------------------------------------------------------------
# syslog
#------------------------------------------------------------------------------
function logmsg()
{
	echo "$1" | logger -t iet-cluster
}

#------------------------------------------------------------------------------
# log ddmap state changes (include hostname name to support node1, node2)
#------------------------------------------------------------------------------
function ddmap_log()
{
	local stamp=$(date +%Y%m%d-%H%M%S)
	local ddmap="$1"
	local msg="$2"
	
	local ddmap_file="$VGMETA_DIR/$ddmap.log"
	echo "$stamp $HOSTNAME $msg" >> $ddmap_file
}

#------------------------------------------------------------------------------
# change ddmap state
#------------------------------------------------------------------------------
function ddmap_state_set()
{
	local ddmap="$1"
	local state="$2"
	
	local ddmap_file="$VGMETA_DIR/$ddmap.state"
	echo -n "$state" > $ddmap_file
	ddmap_log "$ddmap" "state_set=$state"
}

#------------------------------------------------------------------------------
# get ddmap state
#------------------------------------------------------------------------------
function ddmap_state_get()
{
	local ddmap="$1"
	
	local ddmap_file="$VGMETA_DIR/$ddmap.state"
	local state="DDMAP_UNKNOWN"
	if [ -e "$ddmap_file" ]; then
		state=$(cat "$ddmap_file")
	fi
	ddmap_log "$ddmap" "state_get=$state"
	echo "$state"
}

#------------------------------------------------------------------------------
# iptables block iscsi
#------------------------------------------------------------------------------
function iptables_block_iscsi
{
	local ip_spec="$1"
	
	#
	# disallow iscsi traffic in
	#
	logmsg "blocking incoming iscsi network traffic ip_spec: $ip_spec"
	iptables -D INPUT  -p tcp -s $ip_spec --dport $IPTABLES_PORT -j DROP 2> /dev/null
	iptables -I INPUT  -p tcp -s $ip_spec --dport $IPTABLES_PORT -j DROP
	#iptables -D INPUT  -s $ip_spec -j DROP 2> /dev/null
	#iptables -I INPUT  -s $ip_spec -j DROP

	#
	# disallow iscsi traffic out
	#
	logmsg "blocking outgoing iscsi network traffic ip_spec: $ip_spec"
	iptables -D OUTPUT -p tcp -d $ip_spec --sport $IPTABLES_PORT -j DROP 2> /dev/null
	iptables -I OUTPUT -p tcp -d $ip_spec --sport $IPTABLES_PORT -j DROP
	#iptables -D OUTPUT -d $ip_spec -j DROP 2> /dev/null
	#iptables -I OUTPUT -d $ip_spec -j DROP

	#sleep 1
}

#------------------------------------------------------------------------------
# iptables unblock iscsi
#------------------------------------------------------------------------------
function iptables_unblock_iscsi
{
	local ip_spec="$1"
	
	#
	# allow iscsi traffic in and out
	#
	logmsg "allowing iscsi network traffic ip_spec: $ip_spec"
	iptables -D INPUT  -p tcp -s $ip_spec --dport $IPTABLES_PORT -j DROP 2> /dev/null
	iptables -D OUTPUT -p tcp -d $ip_spec --sport $IPTABLES_PORT -j DROP 2> /dev/null
	#iptables -D INPUT  -s $ip_spec -j DROP 2> /dev/null
	#iptables -D OUTPUT -d $ip_spec -j DROP 2> /dev/null
}


#------------------------------------------------------------------------------
# get active volume groups on a given host
#------------------------------------------------------------------------------
function vg_active_get()
{
	local host=$1
	local host_current=$(uname -n)
	local host_wrapper=""
	if [ $host_current != $host ]; then
		host_wrapper="ssh $host"
	fi
	local vg
	for vg in $IET_VGS; do
		$host_wrapper test -d /dev/$vg
		ccode=$?
		if [ $ccode -eq 0 ]; then
			echo $vg
		fi
	done
}

#------------------------------------------------------------------------------
# get lvm slice size (return NULL if undetermined)
#------------------------------------------------------------------------------
function lv_size_get
{
	local lv_name="$1"
	local cmd
	local lv_size
	local ccode
	
	cmd="lvs $lv_name -o lv_size --noheadings --units g"
	lv_size=$($cmd 2> /dev/null | awk '{print $1}')
	if [ -z "$lv_size" ]; then
		lv_size="NULL"
	fi
	echo "$lv_size"
}

#------------------------------------------------------------------------------
# track lun size (get)
#------------------------------------------------------------------------------
function lv_track_lun_size_set
{
	local vg="$1"
	local lv_name="$2"
	local lv_size="$3"
	
	file=$(echo $lv_name | sed -e 's/\//_/g')
	echo "$lv_size" > $VGMETA_DIR/$vg/lunsize/$file
}

#------------------------------------------------------------------------------
# track lun size (set)
#------------------------------------------------------------------------------
function track_lun_size_get
{
	local vg="$1"
	local lv_name="$2"
	
	file=$(echo $lv_name | sed -e 's/\//_/g')
	if [ -e $VGMETA_DIR/$vg/lunsize/$file ]; then
		cat $VGMETA_DIR/$vg/lunsize/$file
	fi
}

#------------------------------------------------------------------------------
# get target tid from iet's /proc/net/iet/...
#------------------------------------------------------------------------------
function target_tid
{
	local tgt_name="$1"

	$ROOT/iet-tools.plx -c target_tid -t "$tgt_name"
}

#------------------------------------------------------------------------------
# check the device mapper state of a given device
#------------------------------------------------------------------------------
function dm_state_check
{
	local device="$1"
	local state="$2"

	local active_state=$(dmsetup info $device | awk '/^State/{print $2}')
	echo "dm> device: $device state: $active_state (expected $state)"
	if [ "$state" != "$active_state" ]; then
		echo "dm_err> state mismatch, unable to proceed"
		exit 1
	fi
}
