#!/bin/sh

#
# Single subnet multi interface routing support
#
# Steffen Plotner, 2010
#

. /etc/iscsi-target/ietd_config.sh

#------------------------------------------------------------------------------
# configure source routing for a network interface
#------------------------------------------------------------------------------
function route_config
{
	#
	# variables used here are sourced from the network interface
	#
	eval `/bin/ipcalc --network --prefix $IPADDR $NETMASK`
	
	#
	# create a custom route table for this interface
	#
	cmd="ip route add $NETWORK/$PREFIX dev $DEVICE src $IPADDR table $IET_ROUTE_TABLE"
	logmsg "route> $cmd"
	$cmd 2> /dev/null
	
	#
	# remove the existing route from the main routing table
	#
	#cmd="ip route del $NETWORK/$PREFIX dev $DEVICE src $IPADDR table main"
	#logmsg "route> $cmd"
	#$cmd 2> /dev/null
	#
	# DON'T DO THAT! If you do, then the host itself cannot
	# originate a connection out those interfaces. And that is
	# very confusing, so the main routing table must keep those
	# entries.
	#
	
	#
	# create a routing rule (to allow this to be re-run
	# first delete the entry and then create it)
	#
	cmd="ip rule del from $IPADDR table $IET_ROUTE_TABLE priority 10"
	logmsg "route> $cmd"
	$cmd 2> /dev/null
	cmd="ip rule add from $IPADDR table $IET_ROUTE_TABLE priority 10"
	logmsg "route> $cmd"
	$cmd
}

#------------------------------------------------------------------------------
# display route status
#------------------------------------------------------------------------------
function route_status()
{
	cmd="ip route show table $IET_ROUTE_TABLE"
	echo "$cmd"
	$cmd
	echo
}

#------------------------------------------------------------------------------
# action handler
#------------------------------------------------------------------------------
function action()
{
	action="$1"
	logmsg "route> enumerating interfaces with action: $action"
	
	if [ $action = "start" ]; then
		echo -n "Starting iSCSI Target: routing"
	fi
	
	for ifname in /etc/sysconfig/network-scripts/ifcfg-eth*; do
		#
		# include interface config, for those that have a custom
		# route table name configure them
		#
		IET_ROUTE_TABLE=""
		. $ifname
		if [ -n "$IET_ROUTE_TABLE" ]; then
			logmsg "route> $ifname has $IET_ROUTE_TABLE"
			if [ $action = "start" ]; then
				route_config
			else
				route_status
			fi
		fi
	done

	if [ $action = "status" ]; then
		cmd="ip rule list"
		echo "$cmd"
		$cmd
	fi

	if [ $action = "start" ]; then
		#
		# remove cached entries
		#
		cmd="ip route flush cache"
		logmsg "route> $cmd"
		$cmd
		
		echo_success
		echo
	fi
}

case "$1" in
	start)
		action start
		;;
	status)
		action status
		;;
	*)
		echo $"Usage: $0 {start|status}"
		exit 1
esac
exit 0
