#!/bin/sh

#
# ietd service pre/post actions
#
# the standard /etc/rc.d/init.d/iscsi-target script invokes
# these ietd_start/stop pre/post functions in which you can
# control what to do before/after specific ietd startup
# and shutdown functions. This allows us to keep the patching
# of the ietd provided init.d script to a minimum.
#

. /etc/iscsi-target/ietd_lib.sh

IETD_HOOKS=1

#
# scripts that use the internal start/stop concept should NOT
# be invoked by a human!
#

function ietd_start_pre()
{
	#
	# we are now checking the existing of the ietd process, we
	# used to check for /proc/net/iet and that is wrong, since
	# the kernel module might not have unloaded previously and a sub-
	# sequent restart will then fail - ouch - not good!
	#
	iet_pid=$(pidof ietd)
	if [ -n "$iet_pid" ]; then
		#
		# if we would let this go, the ddmap would turn
		# every active map to dirty(!)
		#
		echo "ietd already running, refusing to do this!"
		exit 1
	fi
	/etc/iscsi-target/ietd_routing.sh start
	/etc/iscsi-target/ietd_ddmap.sh start_internal
	/etc/iscsi-target/ietd_targets.sh start_internal
	rm /etc/iscsi-target/iet_state/iqn.* 2> /dev/null
}

function ietd_start_post()
{
	/etc/iscsi-target/ietd_iptables.sh start
}

function ietd_stop_pre()
{
	if [ ! -d /proc/net/iet ]; then
		echo "ietd not running, refusing to do this!"
		exit 1
	fi
	/etc/iscsi-target/ietd_iptables.sh stop
	/etc/iscsi-target/ietd_ddmap.sh stop_internal

	#
	# found out that ietadm --op delete does nothing. The killall ietd
	# triggers "Removing all connections, sessions and targets", but
	# it only does the  deletes targets if sessions are removed.
	#
	# So, we remove all sessions, all luns of each target and each target
	# That way, the shutdown should be always clean
	#
	/etc/iscsi-target/iet-tools.plx -c all_sessions_remove
	sleep 4

	logmsg "stop_pre> contents of /proc/net/iet/volume"
	out=$(cat /proc/net/iet/volume)
	logmsg "$out"

	logmsg "stop_pre> contents of /proc/net/iet/session"
	out=$(cat /proc/net/iet/session)
	logmsg "$out"
}

function ietd_stop_post()
{
	sleep 4
}
