#!/bin/sh

#
# lvm multi-host support (this is not LVM clustering!)
#
# This tool accept a list of known lvm commands, parses some of them
# to determine on which host the command should be executed (based
# on which host has which volume group active)
#
# Steffen Plotner, 2010
#

. /etc/iscsi-target/ietd_config.sh

#------------------------------------------------------------------------------
# execute vgs, lvs and only query volume groups that are actively mounted
#------------------------------------------------------------------------------
function lvm_vgs_lvs()
{
	local cmd=$1
	shift

	echo
	local host_current=$(uname -n)
	local host_wrapper=""
	local host
	local vgs
	local vg
	for host in $IET_TARGET_HOSTS; do
		echo "iet_host: $host"
		
		if [ $host_current != $host ]; then
			host_wrapper="ssh $host"
		fi
		
		vgs=$(vg_active_get $host)
		for vg in $vgs; do
			$host_wrapper $cmd $* $vg
		done
		echo
	done
}

#------------------------------------------------------------------------------
# provide lvscan output, hide all inactive logical volumes
#------------------------------------------------------------------------------
function lvm_lvscan
{
	local cmd=$1
	shift
	
	echo
	local host_current=$(uname -n)
	local host_wrapper=""
	local host
	for host in $IET_TARGET_HOSTS; do
		echo "iet_host: $host"
		
		if [ $host_current != $host ]; then
			host_wrapper="ssh $host"
		fi
		
		$host_wrapper $cmd | grep '  ACTIVE'
		echo
	done
}

#------------------------------------------------------------------------------
# create a logical volume, use the last argument to determine on which
# host we create this volume. The last argument could be a volume group
# specification or a device spec for snapshots or lvextend.
#
# lvremove would also work, except that ssh and the interactive question
# do not get along on remote hosts, local hosts are fine.
#------------------------------------------------------------------------------
function lvm_lvcreate_extend_remove
{
	local cmd="$1"
	shift
	if [ -z "$1" ]; then
		echo "missing arguments for command $cmd"
		exit 1
	fi

	eval last=\${$#}

	local host_current=$(uname -n)
	local host_wrapper=""
	local host
	local found=0
	for host in $IET_TARGET_HOSTS; do
		
		if [ $host_current != $host ]; then
			host_wrapper="ssh -t $host"
		fi
		
		$host_wrapper test -b $last
		ccode=$?
		if [ $ccode -eq 0 ]; then
			echo "iet_host: $host ($last)"
			$host_wrapper $cmd $*
			found=1
			break
		fi
		
		$host_wrapper test -d /dev/$last
		ccode=$?
		if [ $ccode -eq 0 ]; then
			echo "iet_host: $host ($last)"
			$host_wrapper $cmd $*
			found=1
			break
		fi
	done
	if [ $found -eq 0 ]; then
		echo "unable to find a host to execute the command on, presumably syntax problem!"
		exit 1
	fi
}

cmd="$1"
shift
case "$cmd" in
	vgs|vgdisplay|lvs)
		lvm_vgs_lvs $cmd $*
		;;
	lvscan)
		lvm_lvscan $cmd $*
		;;
	lvcreate|lvextend|lvremove)
		lvm_lvcreate_extend_remove $cmd $*
		;;		
	*)
		echo "Usage: $0 {vgs|vgdisplay|lvs|lvscan|lvcreate|lvextend|lvremove}"
		exit 1
esac
