#!/bin/sh

#
# session top tool
#
# Steffen Plotner, 2010
#

. /etc/iscsi-target/ietd_config.sh

#------------------------------------------------------------------------------
# save current target session counters into /mnt/iscsi-target to ensure
# that session counters are floated from one cluster node to another during
# failover.
#------------------------------------------------------------------------------
function session_top_save
{
	echo "Saving current target session counters..."
	
	#
	# remove current target_session data
	#
	local vgs=$(vg_active_get $host)
	for vg in $vgs; do
		rm "$VGMETA_DIR/$vg/target_sessions"
	done

	#
	# the output of target sessions is: target<tab>sessions
	#
	$ROOT/iet-tools.plx -c target_sessions | while read line; do
		
		local tgt_name=$(echo "$line" | cut -f1)
		local sessions=$(echo "$line" | cut -f2)
		
		echo "target: $tgt_name sessions: $sessions"
		
		#
		# given a target, we query lun 0 to determine which volume
		# group the lun belongs so that we can save the session count
		# in the appropriate /mnt/iscsi-target/vg_satabeast?/sessions/
		#
		local lun=0
		local lun_path=$($ROOT/iet-tools.plx -c target_lun_path -t $tgt_name -l $lun)
		
		echo " lun_path: $lun_path"
		
		#
		# lvm block device (expected 3 arguments, /dev/vg/lv)
		#
		local dev=$(echo "$lun_path" | cut -f2 -d/)
		local vg=$(echo "$lun_path" | cut -f3 -d/)
		local lv=$(echo "$lun_path" | cut -f4 -d/)
		if [ "$dev" != "dev" -o -z "$vg" -o -z "$lv" ]; then
			echo "unsupported block device name, got $lun_path, requires /dev/vg/lv"
			continue
		fi
		
		#
		# save the session counter for the target
		#
		echo -e "$tgt_name\t$sessions" >> "$VGMETA_DIR/$vg/target_sessions"
	done
	
	echo "Complete."
}

#------------------------------------------------------------------------------
# enum all session files under /mnt/iscsi-target and compare them with the
# running system.
#------------------------------------------------------------------------------
function session_top_diff
{
	#
	# numerate currently active volume groups for previously save target
	# session info and output that into target_sessions_diff for processing
	#
	local vgs=$(vg_active_get $host)
	for vg in $vgs; do
		if [ -e "$VGMETA_DIR/$vg/target_sessions" ]; then
			cat "$VGMETA_DIR/$vg/target_sessions"
		fi
	done | sort | $ROOT/iet-tools.plx -c target_sessions_diff
}

#
# initialize global variables
#
host=$(uname -n)

cmd="$1"
shift
case "$cmd" in
	save)
		session_top_save
		;;
	help|-?|-h)
		echo "$0	displays target session counters compared to previously saved values"
		echo "$0 save	saves current target sessions counts"
		;;
	diff)
		session_top_diff
		;;
	*)
		watch -d $0 diff
		;;
esac
exit 0
